#!/usr/bin/env python

# undocumented option --log, since it's broken
# undocumented option --np, since it's broken
# documented --sync-build
# added hidden option -a (to make -ba work)

# to do (search for "needs work")
#   what should sage file.spkg do?

from subprocess import call
from optparse import OptionParser, OptionGroup, SUPPRESS_HELP, OptionContainer
import os, sys

def help_message(option, opt_str, value, parser):
    """
    Print a help message for Sage, and then exit.  First print an
    abbreviated Sage banner showing the version number.  Then if
    called via "-h" or its synonyms, print a short message by calling
    ``parser.print_some_help``.  If called via "--advanced", print all
    options by calling ``parser.print_help``.
    """
    SAGE_LOCAL_BIN = os.path.join(os.environ['SAGE_LOCAL'], 'bin')
    banner = open(os.path.join(SAGE_LOCAL_BIN, 'sage-banner')).read().splitlines()
    out = [l for l in banner if l.find('ersion') != -1]
    out = [l.replace("*", "|") for l in out]
    print "----------------------------------------------------------------------"
    print "\n".join(out)
    print "----------------------------------------------------------------------"
    print
    if opt_str == "--help" or opt_str == "-h" or opt_str == "-?":
        parser.print_some_help()
    elif opt_str == "--advanced":
        parser.print_help()
    sys.exit(0)

def get_optional_arg(option, opt_str, value, parser):
    """
    If ``value`` is there, assign it to ``option.dest``.  Otherwise,
    look at the next argument: look at the first entry in
    ``parser.rargs``: if it starts with a hyphen, ignore it and set
    ``option.dest`` to True: the option was invoked but with no
    argument.  If the first entry of ``parser.rargs`` doesn't start
    with a hyphen, take it as the argument: assign it to
    ``option.dest``.
    """
    if not value and parser.rargs:
        s = parser.rargs[0]
        if not s.startswith("-"):
            value = s
            del parser.rargs[0]
    if value:
        setattr(parser.values, option.dest, value)
    else:
        setattr(parser.values, option.dest, True)

# this is not currently in use.
def get_several_args(option, opt_str, value, parser):
    """
    Look at ``parser.rargs``: as long as the entries don't start with
    a hyphen, grab them and store them in ``option.dest``.  Stop at
    the first entry starting with a hyphen.
    """
    assert value is None
    value = []

    for arg in parser.rargs:
        # stop on any arg starting with a hyphen
        if arg[:1] == "-":
            break
        value.append(arg)

    del parser.rargs[:len(value)]
    setattr(parser.values, option.dest, value)

def get_rest_of_line(option, opt_str, value, parser):
    """
    Grab the remainder of the line as a list of arguments to be stored
    in ``option.dest``.  If the rest of the line is empty, set
    ``option.dest`` to be an empty list.

    .. warning::

      If ``option.dest`` is an empty list, then ``if option.dest``
      will fail, so instead test ``if isinstance(option.dest, list)``
      to see if this option has been set.
    """
    if not value:
        value = []
    else:
        value = [value]
    while parser.rargs:
        value.append(parser.rargs[0])
        del parser.rargs[0]
    setattr(parser.values, option.dest, value)

class myOptionParser(OptionParser):
    def format_some_option_help(self, formatter=None, keep=None):
        """
        Like the method "format_option_help" from OptionParser, but
        only prints options from the basic parser and from any group
        whose title matches an entry in the list ``keep``.
        """
        if formatter is None:
            formatter = self.formatter
        formatter.store_option_strings(self)
        result = []
        result.append(formatter.format_heading("Options"))
        formatter.indent()
        if self.option_list:
            result.append(OptionContainer.format_option_help(self, formatter))
            result.append("\n")
        for group in self.option_groups:
            if group.title in keep:
                result.append(group.format_help(formatter))
                result.append("\n")
        formatter.dedent()
        # Drop the last "\n", or the header if no options or option groups:
        return "".join(result[:-1])

    def print_some_help(self, file=None):
        """print_help(file : file = stdout)

        Print an extended help message, listing all options and any
        help text provided with them, to 'file' (default stdout).

        (This is the same as the function "print_help" from the
        OptionParser class, except that it calls "format_some_help"
        instead of "format_help".)
        """
        if file is None:
            file = sys.stdout
        encoding = self._get_encoding(file)
        file.write(self.format_some_help().encode(encoding, "replace"))

    def format_some_help(self, formatter=None):
        """
        This is the same as the function "format_help" from the
        OptionParser class, except that it calls
        "format_some_option_help" instead of "format_option_help".  It
        passes the argument keep=['Testing options'], specifying that
        it should only print the basic options and options in the
        group with name 'Testing options', skipping all other groups.
        """
        if formatter is None:
            formatter = self.formatter
        result = []
        if self.usage:
            result.append(self.get_usage() + "\n")
        if self.description:
            result.append(self.format_description(formatter) + "\n")
        result.append(self.format_some_option_help(formatter, keep=['Testing options']))
        result.append(self.format_epilog(formatter))
        return "".join(result)

def setup_parser():
    """
    Set up the command-line parser for Sage.
    """
    # usage message, printed at the top of every help message:
    usage = """Usage: sage [options] [optional-file]
If optional file is present, run Sage on that file; otherwise, start
Sage's command-line interpreter."""
    parser = myOptionParser(prog="Sage", add_help_option=False, usage=usage)
    # add parser groups for the help messages: first, include only the
    # basic parser group and "Testing options" in basic help.  Include everything 
    # in advanced help, using the groups to organize all of the options.
    testing = OptionGroup(parser, "Testing options",
                          "Options used with '--test'.  See also '--optional' and the advanced options '--testall' and '--tnew', as well as the debugging options.")
    advanced = OptionGroup(parser, "Advanced options")
    build = OptionGroup(parser, "Build options", "Options used for cloning and building the Sage library, producing binary or source distributions, etc.")
    other = OptionGroup(parser, "Running other programs", "Sage installs many other programs automatically, like Gap and Mercurial, and others may be installed via optional or experimental packages.  These options provide access to some of these programs.")
    misc = OptionGroup(parser, "Miscellaneous other options")
    debugging = OptionGroup(parser, "Debugging options", "These may also be used with '--test', in which case they get passed on as arguments to 'sage-test'.")

    # In the help messages, the options are printed in the order that they
    # are added to the parser or the group.

    # standard options:
    parser.add_option("-h", "-?", "--help", action="callback",
                      dest='help',
                      callback=help_message,
                      help="show a help message and exit")
    parser.add_option("-v", "--version", action="store_true",
                      help="show version number and exit")
    parser.add_option("--advanced", action="callback",
                      callback=help_message,
                      dest='advanced',
                      help="list advanced options and exit")
    parser.add_option("-n", "--notebook", action="callback", dest="notebook",
                      callback=get_rest_of_line,
                      help="start the Sage notebook, passing all remaining arguments to the 'notebook' command in Sage")
    parser.add_option("-c", "--cmd", "--command", action="store_true",
                      help="evaluate CMD (= all remaining arguments) as Sage code and exit")
    parser.add_option("-i", "--install", action="callback", dest="install",
                      callback=get_optional_arg,
                      help="install the given Sage packages; if no packages given, print the installed packages")
    parser.add_option("--optional", action="store_true",
                      help="by itself: print all optional packages that can be downloaded; with '--test': test all examples marked with '# optional'")
    parser.add_option("--experimental", action="store_true",
                      help="print all experimental packages that can be downloaded")
    parser.add_option("--docbuild", action="callback", dest="docbuild",
                      callback=get_rest_of_line,
                      help="build the documentation using 'sage --docbuild <document> <format>'; type 'sage --docbuild' with no arguments for more information")
    parser.add_option("-t", "--test", action="callback", dest="test",
                      callback=get_optional_arg,
                      metavar="[list of files or directories]",
                      help="test examples in .py, .pyx, .sage or .tex files")

    # testing options:
    testing.add_option("--long", action="store_true",
                       help="include lines with the phrase 'long time'")
    testing.add_option("--only-optional", "--only_optional", metavar="TAGS",
                       action="store", dest="only_optional", 
                       help="specify TAGS as a list of the form 'database_gap,dot2tex,chomp', and run only those doctests labeled with '#optional <nonempty subset of TAGS>'")
    testing.add_option("-p", "--parallel", action="store",
                       metavar="N",
                       help="test in parallel using N threads, with 0 interpreted as minimum(8, cpu_count())")
    testing.add_option("--randorder", metavar="SEED", action="callback",
                       dest="randorder", callback=get_optional_arg,
                       help="randomize order of tests, optionally specifying SEED")
    testing.add_option("--sagenb", action="store_true",
                       help="test all sagenb files")
    testing.add_option("--verbose", action="store_true",
                       help="print debugging output during the test")
    testing.add_option('--force-lib', '--force_lib', action="store_true",
                       help="assume all files are Sage library files, regardless of location")

    # advanced options:
    advanced.add_option("-s", "--save", action="callback", dest="save",
                        callback=get_optional_arg,
                        help="used with --install: install the packages, but do not delete their spkg/build directories")
    advanced.add_option("-f", "--force", "--force-install", action="callback",
                        callback=get_optional_arg, dest='force',
                        help="force build of the given Sage package (even if already installed); may also be used with --build-all")
    advanced.add_option("--testall", action="store_true",
                        help="test all source, docs, and examples in your Sage distro.")
    advanced.add_option("--testnew", "--tnew", action="store_true",
                        help="like --testall, but only tests files modified since last commit")
    advanced.add_option("--upgrade", action="callback", metavar="URL",
                        dest="upgrade", callback=get_optional_arg,
                        help="download, build and install standard packages from URL.  If URL not given, automatically selects a suiteable mirror.  If URL='ask', it lets you select the mirror (uses SAGE_SERVER as default).")
    advanced.add_option("--root", action="store_true",
                        help="print the Sage root directory")
    advanced.add_option("--standard", action="store_true",
                        help="print all standard packages that can be downloaded")
    advanced.add_option("--preparse", action="store", metavar="FILE",
                        help="preparse file.sage, producing corresponding file.sage.py")

    # "build" options
    build.add_option("--branch", dest="print_branch", action="store_true",
                     help="print the current Sage branch")
    build.add_option("--clone", action="store", metavar="BRANCH",
                     help="clone a new branch of the Sage library from the current branch")
    build.add_option("-r", "--run", dest="run", action="callback",
                     callback=get_optional_arg,
                     help="with argument BRANCH, switch to BRANCH and run Sage; with no argument, run Sage; may be combined with -b")
    build.add_option("-b", "--build", dest="build", action="callback", 
                     callback=get_optional_arg,
                     help="with argument BRANCH, switch to and build Sage branch in devel/sage-BRANCH; with no argument, build current branch")
    build.add_option("--build-all", "--ba", dest="build_all",
                     action="callback", callback=get_optional_arg,
                     help="with argument BRANCH, switch to BRANCH and rebuild all Cython code there; with no argument, rebuild all Cython code in current branch; can instead use -ba or -b -a")
    # -a is present so -ba and -baf work.
    build.add_option("-a", "--all", dest="all", action="store_true",
                     help=SUPPRESS_HELP)
    build.add_option("--build-all-force", "--ba-force", dest="build_all_force",
                     action="callback", callback=get_optional_arg,
                     help="same as --build-all but don't query before rebuilding; can instead use -baf or -b -a -f")
    build.add_option("--pkg", "--spkg", action="store", metavar="DIR",
                     help="create Sage package dir.spkg from DIR (bzip2 compression)")
    build.add_option("--pkg-nc", "--pkg_nc", "--spkg-nc", "--spkg_nc",
                     action="store", metavar="DIR",
                     help="create Sage package dir.spkg from DIR (no compression)")
    build.add_option("--bdist", action="store", metavar="VER",
                     help="build a binary distribution of Sage, version VER")
    build.add_option("--sdist", action="store", metavar="VER",
                     help="build a source distribution of Sage, version VER")
    build.add_option("--sync-build", action="callback", dest="sync_build",
                     callback=get_rest_of_line,
                     help="delete any files in $SAGE_DEVEL/sage/build/ which don't have a corresponding source file in $SAGE_DEVEL/sage/sage/.")
    build.add_option("--merge", action="callback", dest="merge",
                     callback=get_rest_of_line,
                     help="run Sage's automatic merge and test script, passing it all remaining arguments")

    # miscellaneous options
    misc.add_option("--coverage", action="store", metavar="FILES",
                    help="give info about doctest coverage of FILES")
    misc.add_option("--coverageall", action="store_true",
                    help="give summary info about doctest coverage of all files in the Sage library")
    misc.add_option("--fixdoctests", action="store",
                    metavar="FILE",
                    help="create <file.py>.out that would pass the doctests, and output a patch")
    misc.add_option("--grep", action="callback", metavar="STRING",
                    dest='grep', callback=get_rest_of_line,
                    help="grep through all the Sage library code for STRING")
    misc.add_option("--grepdoc", action="callback", metavar="STRING",
                    dest='grepdoc', callback=get_rest_of_line,
                    help="grep through all the Sage documentation for STRING")
    misc.add_option("--startuptime", action="store_true",
                    help="display how long each component of Sage takes to startup")
    misc.add_option("--cleaner", action="store_true", help="run the Sage cleaner")
    misc.add_option("--crap", action="store", metavar="TARFILE",
                    help="detect suspicious garbage in a Sage source distribution")
    misc.add_option("--inotebook", action="callback", dest="inotebook",
                    callback=get_rest_of_line,
                    help="start the *insecure* Sage notebook, passing all remaining arguments to the 'notebook' command in Sage")
    misc.add_option("--log", action="store_true",
                    help=SUPPRESS_HELP)  # currently broken
#                    help="add entry to <SAGE_ROOT>/changelog.txt")
    misc.add_option("--min", action="store_true",
                    help="do not populate the global namespace")
    misc.add_option("--np", "--no-prompts", action="store_true",
                    dest="no_prompts",
                    help=SUPPRESS_HELP)  # currently broken
#                    help="run with no output prompts (useful for making doctests)")
    misc.add_option("-q", "--quiet", action="store_true", help="start with no banner")
    misc.add_option("--gthread", action="store_true",
                    help="pass this option through to ipython")
    misc.add_option("--qthread", action="store_true",
                    help="pass this option through to ipython")
    misc.add_option("--q4thread", action="store_true",
                    help="pass this option through to ipython")
    misc.add_option("--wthread", action="store_true",
                    help="pass this option through to ipython")
    misc.add_option("--pylab", action="store_true",
                    help="pass this option through to ipython")

    # running other programs -- many of these options are taken care
    # of in sage-sage-quickstart, not here, but we document them here,
    # and we alse keep the functionality, just in case.
    other.add_option("--sh", "--shell", action="store_true",
                        help="run $SHELL (%s) with Sage environment variables set" % os.environ['SHELL'])
    other.add_option("--axiom", action="callback", dest="axiom",
                     callback=get_rest_of_line,
                     help="run Sage's axiom, passing it all remaining arguments")
    other.add_option("--combinat", action="callback", dest="combinat",
                     callback=get_rest_of_line,
                     help="run sage-combinat patch management script, passing it all remaining arguments")
    other.add_option("--cython", "--pyrex", action="callback", dest="cython",
                     callback=get_rest_of_line,
                     help="run Cython, passing it all remaining arguments")
    other.add_option("--ecl", "--lisp", action="callback", dest="ecl",
                     callback=get_rest_of_line,
                     help="run Common Lisp, passing it all remaining arguments")
    other.add_option("--gap", action="callback", dest="gap",
                     callback=get_rest_of_line,
                     help="run Sage's Gap, passing it all remaining arguments")
    other.add_option("--gdb-ipython", action="callback", dest="gdb_ipython",
                     callback=get_rest_of_line,
                     help="run Sage's IPython under the control of gdb")
    other.add_option("--gp", action="callback", dest="gp",
                     callback=get_rest_of_line,
                     help="run Sage's GP (Pari), passing it all remaining arguments")
    other.add_option("--hg", "--mercurial", action="callback", dest="hg",
                     callback=get_rest_of_line,
                     help="run Sage's hg (Mercurial), passing it all remaining arguments")
    other.add_option("--hgmerge", action="callback", dest="hgmerge",
                     callback=get_rest_of_line,
                     help="run hgmerge, passing it all remaining arguments")
    other.add_option("--ipython", action="callback", dest="ipython",
                     callback=get_rest_of_line,
                     help="run Sage's Ipython using the default environment (not Sage), passing it all remaining arguments")
    other.add_option("--kash", action="callback", dest="kash",
                     callback=get_rest_of_line,
                     help="run Sage's Kash, passing it all remaining arguments")
    other.add_option("--M2", action="callback", dest="M2",
                     callback=get_rest_of_line,
                     help="run Sage's Macaulay2, passing it all remaining arguments")
    other.add_option("--maxima", action="callback", dest="maxima",
                     callback=get_rest_of_line,
                     help="run Sage's Maxima, passing it all remaining arguments")
    other.add_option("--mwrank", action="callback", dest="mwrank",
                     callback=get_rest_of_line,
                     help="run Sage's mwrank, passing it all remaining arguments")
    other.add_option("--python", action="callback", dest="python",
                     callback=get_rest_of_line,
                     help="run Sage's python interpreter, passing it all remaining arguments")
    other.add_option("-R", "--R", action="callback", dest="R",
                     callback=get_rest_of_line,
                     help="run Sage's R, passing it all remaining arguments")
    other.add_option("--scons", action="callback", dest="scons",
                     callback=get_rest_of_line,
                     help="run Sage's scons, passing it all remaining arguments")
    other.add_option("--singular", action="callback", dest="singular",
                     callback=get_rest_of_line,
                     help="run Sage's singular, passing it all remaining arguments")
    other.add_option("--twistd", action="callback", dest="twistd",
                     callback=get_rest_of_line,
                     help="run Sage's twistd server, passing it all remaining arguments")
    other.add_option("--wiki", action="callback", dest="wiki",
                     callback=get_rest_of_line,
                     help="run Sage's Moin Moin Wiki server, passing it all remaining arguments")

    # debugging options
    debugging.add_option("--gdb", dest="gdb", action="store_true",
                         help="run Sage under the control of gdb")
    debugging.add_option("--memcheck", action="store_true",
                        help="run Sage under the control of valgrind using the memcheck tool. The log files can be found in $HOME/.sage/ -- they are named sage-memcheck.PID.  This is only available if valgrind is installed.")
    debugging.add_option("--valgrind", action="store_true",
                        help="same as --memcheck")
    debugging.add_option("--massif", action="store_true",
                        help="like --memcheck, but use massif instead of memcheck")
    debugging.add_option("--cachegrind", action="store_true",
                        help="like --memcheck, but use cachegrind instead of memcheck")
    debugging.add_option("--callgrind", action="store_true",
                        help="like --memcheck, but use callgrind instead of memcheck")
    debugging.add_option("--omega", action="store_true",
                        help="like --memcheck, but use omega instead of memcheck")

    # add these groups in the order they should appear in the help messages
    parser.add_option_group(testing)
    parser.add_option_group(advanced)
    parser.add_option_group(build)
    parser.add_option_group(other)
    parser.add_option_group(debugging)
    parser.add_option_group(misc)
    return parser

def sage_setup():
    import shutil
    SAGE_LOCAL = os.environ['SAGE_LOCAL']
    if os.environ.get('SAGE_BANNER', 'yes') != 'no':
        print open(os.path.join(SAGE_LOCAL_BIN, 'sage-banner')).read(),
    if call([os.path.join(SAGE_LOCAL, 'bin', 'sage-location')]):
        print "Error running 'sage-location'"
        return 1
    os.environ['IPYTHONDIR'] = os.path.join(os.environ['DOT_SAGE'], 'ipython')
    os.environ['IPYTHONRC'] = "ipythonrc"
    if not os.path.exists(os.environ['IPYTHONDIR']):
        shutil.copytree(os.path.join(os.environ['SAGE_ROOT'], 'ipython'),
                        os.environ['DOT_SAGE'])
    call(os.path.join(SAGE_LOCAL, 'bin', 'sage-cleaner') + " &", shell=True)
    return

def sage(*args):
    if sage_setup() == 1:
        print "Error setting up Sage."
        sys.exit(1)
    return call([os.path.join(os.environ['SAGE_LOCAL'], 'bin', 'sage-ipython')]
                 + list(args))

if __name__ == '__main__':
    # Parse the command-line.
    parser = setup_parser()
    options, args = parser.parse_args()

    if options.quiet:
        os.environ['SAGE_BANNER'] = 'no'
    if options.min:
        os.environ['SAGE_IMPORTALL'] = "no"

    logopt = os.environ.get('LOGOPT', '')
    if options.no_prompts:
        os.environ['LOGOPT'] = "-prompt_out=\\r %s" % logopt

    SAGE_ROOT = os.environ['SAGE_ROOT']
    SAGE_LOCAL = os.environ['SAGE_LOCAL']
    SAGE_LOCAL_BIN = os.path.join(os.environ['SAGE_LOCAL'], 'bin')

    if options.cleaner:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-cleaner')])
        sys.exit(val)

    if options.version:
        banner = open(os.path.join(SAGE_LOCAL_BIN, 'sage-banner')).read().splitlines()
        out = [l for l in banner if l.find('ersion') != -1]
        out = [l.replace("*", "|") for l in out]
        print "----------------------------------------------------------------------"
        print "\n".join(out)
        print "----------------------------------------------------------------------"
        sys.exit(0)

    if options.root:
        print SAGE_ROOT
        sys.exit(0)

    if options.print_branch:
        path = os.path.realpath(os.path.join(SAGE_ROOT, 'devel', 'sage')).rstrip('/')
        print os.path.split(path)[1]
        sys.exit(0)

    # running other programs
    #
    # test each option which uses callback=get_rest_of_line using
    # "isinstance(options.dest, list)": see docstring for
    # get_rest_of_line
    #
    # many of these are actually dealt with in sage-sage-quickstart,
    # so they are redundant here.  we keep the functionality here just
    # in case.
    if isinstance(options.axiom, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'axiom')] + options.axiom
                   + args)
        sys.exit(val)
    if isinstance(options.combinat, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-combinat')]
                   + options.combinat + args)
        sys.exit(val)
    if options.clone:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-clone'), options.clone]
                   + args)
        sys.exit(val)
    if isinstance(options.cython, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'cython')] + options.cython
                   + args)
        sys.exit(val)
    if isinstance(options.ecl, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'ecl')] + options.ecl + args)
        sys.exit(val)
    if isinstance(options.gap, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'gap')] + options.gap + args)
        sys.exit(val)
    if isinstance(options.gdb_ipython, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-gdb-ipython')]
                   + options.gdb_ipython + args)
        sys.exit(val)
    if isinstance(options.gp, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'gp')] + options.gp + args)
        sys.exit(val)
    if isinstance(options.grep, list):
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-grep')] 
                   + options.grep + args)
        sys.exit(val)
    if isinstance(options.grepdoc, list):
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-grepdoc')]
                   + options.grepdoc + args)
        sys.exit(val)
    if isinstance(options.hg, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'hg')] + options.hg + args)
        sys.exit(val)
    if isinstance(options.hgmerge, list):
        # NEED in Sage Environment under OS X
        if 'LIBRARY_PATH' in os.environ:
            del os.environ['LIBRARY_PATH']
        if 'LD_LIBRARY_PATH' in os.environ:
            del os.environ['LD_LIBRARY_PATH']
        if 'DYLD_LIBRARY_PATH' in os.environ:
            del os.environ['DYLD_LIBRARY_PATH']
        val = call([os.path.join(SAGE_LOCAL_BIN, 'hgmerge')]
                   + options.hgmerge + args)
        sys.exit(val)
    if isinstance(options.ipython, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'ipython')]
                   + options.ipython + args)
        sys.exit(val)
    if isinstance(options.kash, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'kash')]
                   + options.kash + args)
        sys.exit(val)
    if options.log:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-log')])
        sys.exit(val)
    if isinstance(options.M2, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'M2')] + options.M2 + args)
        sys.exit(val)
    if isinstance(options.maxima, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'maxima')]
                   + options.maxima + args)
        sys.exit(val)
    if isinstance(options.merge, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-apply-ticket')]
                   + options.merge + args)
        sys.exit(val)
    if isinstance(options.mwrank, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'mwrank')]
                   + options.mwrank + args)
        sys.exit(val)
    if options.preparse:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-preparse'),
                   options.preparse] + args)
        sys.exit(val)
    if isinstance(options.python, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'python')]
                   + options.python + args)
        sys.exit(val)
    if isinstance(options.R, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'R')] + options.R + args)
        sys.exit(val)
    if isinstance(options.scons, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'scons')] + options.scons
                   + args)
        sys.exit(val)
    if isinstance(options.singular, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'singular')]
                   + options.singular + args)
        sys.exit(val)
    if options.startuptime:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'python'),
                    os.path.join(SAGE_LOCAL_BIN, 'sage-startuptime.py')])
        sys.exit(val)
    if isinstance(options.sync_build, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'python'),
                    os.path.join(SAGE_LOCAL_BIN, 'sage-sync-build.py')]
                   + options.syncbuild + args)
        sys.exit(val)
    if isinstance(options.twistd, list):
        val = call(['python', '$(which twistd)'] + options.twistd + args)
        sys.exit(val)
    if isinstance(options.wiki, list):
        val = call([os.path.join(SAGE_LOCAL_BIN, 'wiki')] + options.wiki
                   + args)
        sys.exit(val)

    if options.fixdoctests:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-fixdoctest'),
                    options.fixdoctests] + args)
        sys.exit(val)

    # coverage
    if options.coverage:
        print [os.path.join(SAGE_LOCAL_BIN, 'sage-coverage'),
                    options.coverage] + args
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-coverage'),
                    options.coverage] + args)
        sys.exit(val)
    if options.coverageall:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-coverageall')] + args)
        sys.exit(val)

    # crap
    if options.crap:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-crap'),
                    options.crap] + args)
        sys.exit(val)

    # build
    branch = None
    run = True
    if options.build or options.build_all or options.build_all_force:
        branches = [options.build, options.build_all,
                    options.build_all_force, options.run]
        branch = [a for a in branches if a is not None and a is not True]
        if len(branch) > 0:
            branch = branch[0:1]
        all = (options.all or options.build_all or options.build_all_force
                 or options.force)
        if all:
            branch = ['-b'] + branch
        force_build = options.build_all_force or options.force
        if all and not force_build:
            print """ *** WARNING ***
 You are about to rebuild the entire Sage library.
 This will take a significant amount of time."""
            s = raw_input(" Do you want to proceed? [y/n] ")[0].lower()
            while s != "y" and s != "n":
                s = raw_input(" Choose one of y, n: ")[0].lower()
            force_build = s == "y"
            if not force_build:
                sys.exit(0)
        
        if (all and force_build) or not all:
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-build')] +
                        branch + args)
            if val != 0:
                sys.exit(val)
            if isinstance(options.notebook, list):
                os.environ['SAGE_BANNER'] = 'no'
                if sage_setup() == 1:
                    print "Error setting up Sage."
                    sys.exit(1)
                val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-notebook')]
                           + options.notebook + args)
                sys.exit(val)
            if isinstance(options.inotebook, list):
                os.environ['SAGE_BANNER'] = 'no'
                if sage_setup() == 1:
                    print "Error setting up Sage."
                    sys.exit(1)
                val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-notebook-insecure')]
                           + options.inotebook + args)
                sys.exit(val)
            run = options.run
            if run:
                val = sage(*args)
                sys.exit(val)
    elif options.run:
        # -r with no -b: if a branch is specified, switch to it and run Sage.
        # otherwise, just run Sage.
        if options.run is not True:
            branch = options.run
            devel_sage = os.path.join(SAGE_ROOT, 'devel', 'sage')
            os.chdir(os.path.join(SAGE_ROOT, 'devel'))
            if not os.path.exists(devel_sage + "-" + branch):
                print "No such branch '%s/devel/sage-%s'" % (SAGE_ROOT, branch)
                print "Use 'sage --clone' to create a new branch."
                sys.exit(1)
            if os.path.exists(devel_sage):
                os.remove(devel_sage)
            os.symlink(devel_sage + "-" + branch, devel_sage)

    # sage --sh:
    if options.sh:
        print """
Starting subshell with Sage environment variables set.
Be sure to exit when you are done and do not do anything
with other copies of Sage!
"""
        SHELL_NAME = os.path.split(os.environ['SHELL'])[1]
        print "Bypassing shell configuration files ..."
        print 
        # We must start a new shell with no .profile or .bashrc files
        # processed, so that we know our path is correct
        os.environ['PS1'] = "${PWD}\nsage subshell$ "
        if SHELL_NAME == "bash":
            SHELL_OPTS = "--norc"
        elif SHELL_NAME == "csh":
            SHELL_OPTS = "-f"
        elif SHELL_NAME == "ksh":
            SHELL_OPTS = "-p"
        elif SHELL_NAME == "sh":
            SHELL_OPTS = "--norc"
        elif SHELL_NAME == "tcsh":
            SHELL_OPTS = "-f"
        elif SHELL_NAME == "zsh":
            SHELL_OPTS = "-f -d"
        else:
            print "Unknown shell: %s!" % SHELL_NAME
            print "Exiting Sage."
            sys.exit(1)
        call([SHELL_NAME, SHELL_OPTS] + args)
        print "Exited Sage subshell."
        sys.exit(0)

    # testing
    if options.test:
        # process other arguments
        if options.test is True:
            test_args = args
        else:
            test_args = [options.test] + args
        if options.optional:
            test_args += ["--optional"]
        if options.long:
            test_args += ["--long"]
        if options.only_optional:
            # test_args += ["--only-optional=%s" % options.only_optional]
            test_args += ["--only-optional"]
        if options.randorder:
            if options.randorder is True:
                test_args += ["--randorder"]
            else:
                test_args += ["--randorder %s" % options.randorder]
        if options.sagenb:
            test_args += ["--sagenb"]
        if options.verbose:
            test_args += ["--verbose"]
        if options.gdb:
            test_args += ["--gdb"]
        if options.valgrind:
            test_args += ["--valgrind"]
        if options.memcheck:
            test_args += ["--memcheck"]
        if options.massif:
            test_args += ["--massif"]
        if options.cachegrind:
            test_args += ["--cachegrind"]
        if options.callgrind:
            test_args += ["--callgrind"]
        if options.omega:
            test_args += ["--omega"]
        if options.force_lib:
            test_args += ["--force_lib"]
        # now test
        if options.parallel:
            # for parallel testing, run sage-starts first to make sure
            # all appropriate subdirectories exist.
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-starts')])
            if val != 0:
                print "Error running sage-starts"
                sys.exit(val)
            sage_init = os.path.join(os.environ['DOT_SAGE'], 'init.sage')
            if not os.path.isfile(sage_init):
                print "init.sage does not exist ... creating"
                val = call(['touch', sage_init])
                if val != 0:
                    print "Error creating init.sage"
                    sys.exit(val)
            test_args = [str(options.parallel)] + test_args
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-ptest')] + test_args)
        else:
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-test')] + test_args)
        sys.exit(val)

    if options.testnew:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-test-new')] + args)
        sys.exit(val)
    if options.testall:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-maketest')] + args)
        sys.exit(val)

    # packages.
    if options.standard:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-list-standard')] + args)
        sys.exit(val)
    # the "optional" check has to come after testing:
    # 'sage -t --optional' should pass the '--optional' flag on to sage-test.
    if options.optional:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-list-optional')] + args)
        sys.exit(val)
    if options.experimental:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-list-experimental')] + args)
        sys.exit(val)
    if options.install or options.force:
        packages = [options.install, options.force, options.save]
        pkg = [a for a in packages if a is not None and a is not True]
        if len(pkg) > 0:
            args = [pkg[0]] + args
        else:
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-spkg')])
            sys.exit(val)
        extra = []
        if options.force:
            print "Force installing " + ",".join(args)
            extra += ["-f"]
        else:
            print "Installing " + ",".join(args)
        if options.save:
            extra += ["-s"]
        for pkg in args:
            print "Calling sage-spkg on %s" % pkg
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-spkg')] + extra
                       + [os.path.abspath(pkg)])
            if val != 0:
                sys.exit(val)
        sys.exit(val)

    if options.pkg:
        pkgs = [options.pkg] + args
        for pkg in pkgs:
            print "Creating Sage package %s" % pkg
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-pkg')] + [pkg])
            if val != 0:
                sys.exit(val)
        sys.exit(0)

    if options.pkg_nc:
        pkgs = [options.pkg_nc] + args
        for pkg in pkgs:
            print "Creating Sage package %s with no compression" % pkg
            val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-pkg-nocompress')] + [pkg])
            if val != 0:
                sys.exit(val)
        sys.exit(0)

    if options.sdist:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-sdist'), options.sdist,
                    SAGE_ROOT])
        sys.exit(val)

    if options.bdist:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-bdist'), options.sdist,
                    SAGE_ROOT])
        sys.exit(val)

    # upgrade
    if options.upgrade:
        # People often move the Sage install right before doing the upgrade, so it's
        # important to fix any path hardcoding issues first, or certain library
        # links will fail.
        if call([os.path.join(SAGE_LOCAL, 'bin', 'sage-location')]):
            print "Error running 'sage-location'"
            sys.exit(1)         
        # Do it twice since when installing sage-scripts and a running
        # script changes, it gets confused and exits with an error.
        # Running again (with the script replaced) then fixes the problem.
        # Run from a temporary copy of sage-sage
        if options.upgrade is True:
            url = []
        else:
            url = [options.upgrade]
        val = call([os.path.join(SAGE_LOCAL, 'bin', 'sage-upgrade')] + url)
        if val == 2:   # this exit code means the user elected not to do the upgrade at all.
            sys.exit(2)
        print "Double checking that all packages have been installed."
        val = call([os.path.join(SAGE_LOCAL, 'bin', 'sage-upgrade')] + url)
        sys.exit(val)

    # docbuild
    if isinstance(options.docbuild, list):
        args = options.docbuild + args
        
        val = call([os.path.join(SAGE_LOCAL_BIN, 'python'),
                    os.path.join(SAGE_ROOT, 'devel', 'sage', 'doc',
                                 'common', 'builder.py')]
                   + args)
        sys.exit(val)

    # debugging tools
    # check these options after testing: 'sage -t --valgrind ...'
    # should pass the '--valgrind' option on to sage-test
    if options.gdb:
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-gdb')] + args)
        sys.exit(val)
    if options.memcheck or options.valgrind:
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-valgrind')])
        sys.exit(val)
    if options.massif:
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-massif')])
        sys.exit(val)
    if options.cachegrind:
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-cachegrind')])
        sys.exit(val)
    if options.callgrind:
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-callgrind')])
        sys.exit(val)
    if options.omega:
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-omega')])
        sys.exit(val)

    # run Sage
    if options.cmd:
        os.environ['SAGE_BANNER'] = 'no'
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-eval')]
                   + args)
        sys.exit(val)

    if isinstance(options.notebook, list):
        os.environ['SAGE_BANNER'] = 'no'
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-notebook')]
                   + options.notebook + args)
        sys.exit(val)
    if isinstance(options.inotebook, list):
        os.environ['SAGE_BANNER'] = 'no'
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-notebook-insecure')]
                   + options.inotebook + args)
        sys.exit(val)

    if args:
        CUR = os.environ['CUR']
        val = call([os.path.join(SAGE_LOCAL_BIN, 'sage-run'), CUR] + args)
        sys.exit(val)
    elif run:
        if options.gthread:
            args += ["-gthread"]
        if options.qthread:
            args += ["-qthread"]
        if options.q4thread:
            args += ["-q4thread"]
        if options.wthread:
            args += ["-wthread"]
        if options.pylab:
            args += ["-pylab"]
        if sage_setup() == 1:
            print "Error setting up Sage."
            sys.exit(1)
        # use os.system instead of call because os.system seems to
        # handle ctrl-c better.
        val = os.system(os.path.join(SAGE_LOCAL_BIN, 'sage-ipython') + " -i" + " ".join(args))
        sys.exit(val)
    else:
        sys.exit(0)

    # to implement?  needs work?
    # if [ $# -ge 1 ]; then
    #    T=`echo "$1" | sed -e "s/.*\.//"`
    #    if [ "$T " = "spkg " ]; then
    #        install "" "$@"
    #        exit $?
    #    fi
    #    cd "$SAGE_LOCAL/bin/"
    #    sage-run "$CUR" "$@"
    #    exit $?
    # fi
